//
//  EZVerifyCodeManager.h
//  SecurityCodeSDK
//
//  Created by SUMSL on 2019/1/15.
//  Copyright © 2019 security. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

@protocol EZVerifyCodeManagerDelegate <NSObject>

@optional

/**
 * 验证码组件初始化出错
 *
 * @param error 错误信息
 */
- (void)verifyCodeInitFailed:(NSString *)error;

/**
 * 完成验证之后的回调
 *
 * @param result 验证结果 BOOL:YES(成功)/NO(失败)
 *
 * @param message 结果描述信息
 *
 */
- (void)verifyCodeValidateFinish:(BOOL)result message:(NSString *)message;


/**
 * 关闭验证码窗口后的回调
 */
- (void)verifyCodeCloseWindow;

/**
 * 网络错误
 *
 * @param error 网络错误信息
 */
- (void)verifyCodeNetError:(NSError *)error;

@end


@interface EZVerifyCodeManager : NSObject

/**
 * @abstract    delegate,见EZVerifyCodeManagerDelegate
 */
@property(nonatomic, weak) id<EZVerifyCodeManagerDelegate>delegate;


/**
 * @abstract    验证码图片背景的透明度
 *
 * @说明         范围:0~1，0表示全透明，1表示不透明。默认值:0.3
 */
@property(nonatomic) CGFloat           alpha;

/**
 * @abstract    验证码图片背景的颜色
 *
 * @说明         默认值:黑色
 */
@property(nonatomic) UIColor           *color;

/**
 *  @abstract   单例
 *
 *  @return     返回EZVerifyCodeManager对象
 */
+ (EZVerifyCodeManager *)sharedInstance;

/**
 *  @abstract   配置参数
 *
 *  @param      appKey      key
 *  @param      timeoutInterval 加载验证码的超时时间,这个时间尽量设置长一些，比如5秒以上(5-10s);默认设置5秒
 *
 */
- (void)configureVerifyCode:(NSString *)appKey
                    timeout:(NSTimeInterval)timeoutInterval;

/**
 *  @abstract   展示验证码视图
 *
 *  @说明        展示位置:[[[UIApplication sharedApplication] delegate] window];全屏居中显示;
 */
- (void)openVerifyCodeView;


@end
