//
//  EZVerifyCodeManager.m
//  SecurityCodeSDK
//
//  Created by SUMSL on 2019/1/15.
//  Copyright © 2019 security. All rights reserved.
//

#import "EZVerifyCodeManager.h"

#define EZWIDTH      [UIScreen mainScreen].bounds.size.width
#define EZTHEIGHT    [UIScreen mainScreen].bounds.size.height

#define DEWIDTH      (EZWIDTH/5*4+20)
#define DEHEIGHT     (EZWIDTH/5*2+65+50+50)

@interface EZVerifyCodeManager ()<UIGestureRecognizerDelegate>

/**
 背景
 */
@property (nonatomic,strong)UIView *backView;


/**
 图片view
 */
@property (nonatomic,strong)UIView *centerView;


/**
 key
 */
@property (nonatomic,strong)NSString *appKey;


/**
 token
 */
@property (nonatomic,strong)NSString *clientToken;


/**
 请求超时时长
 */
@property (nonatomic,assign)NSTimeInterval timeoutInterval;


/**
 验证的图片
 */
@property (nonatomic,strong)UIImageView *codeImageView;


/**
 标题
 */
@property (nonatomic,strong)UILabel *titleLabel;

/**
 问题
 */
@property (nonatomic,strong)UILabel *keylabel;

@property (nonatomic,strong)UIImageView *circleImageVIew;


@property (nonatomic,strong)UIImageView *keyImageView;


@property (nonatomic,strong)UIImageView *loadImageView;

@property (nonatomic,strong)UILabel *hudLabel;

@property (nonatomic,assign)CGPoint rightPoint; //正确的点

@property (nonatomic,strong)NSMutableArray *pointArray;

@property (nonatomic,strong)UIImageView *alertImageView;

@property (nonatomic,strong)UILabel *alertlabel;

@property(nonatomic) CGRect            frame;

@property (nonatomic,strong)UIView *topLine;

@property (nonatomic,strong)UIView *bottomLine;

@end


@implementation EZVerifyCodeManager
+ (EZVerifyCodeManager *)sharedInstance{
    static EZVerifyCodeManager *manager = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        manager = [[super allocWithZone:NULL] init];
    });
    return manager;
}
- (instancetype)init{
    if (self = [super init]) {
        self.alpha = 0.3;
        self.frame = CGRectMake((EZWIDTH-DEWIDTH)/2,EZTHEIGHT/2-DEHEIGHT/2, DEWIDTH, DEHEIGHT);
        self.color = [UIColor blackColor];
        self.timeoutInterval = 5;
    }
    return self;
}
- (void)configureVerifyCode:(NSString *)appKey timeout:(NSTimeInterval)timeoutInterval{
    self.appKey = appKey;
    self.timeoutInterval = timeoutInterval;
}
- (void)openVerifyCodeView{
    if (self.appKey.length == 0) {
        if (self.delegate && [self.delegate respondsToSelector:@selector(verifyCodeInitFailed:)]) {
            [self.delegate verifyCodeInitFailed:@"请设置appkey"];
        }
        return;
    }
    
    UIWindow *window = [[[UIApplication sharedApplication] delegate] window];
    self.backView.frame = window.frame;
    self.backView.backgroundColor =  [self.color colorWithAlphaComponent:self.alpha];
    [window addSubview:self.backView];
    
    [self.backView addSubview:self.centerView];
    self.centerView.frame = self.frame;
    
    [self.centerView addSubview:self.titleLabel];
    self.titleLabel.frame = CGRectMake(10, 0, self.centerView.frame.size.width-20, 50);
    self.titleLabel.text = @"";
    
    [self.centerView addSubview:self.keyImageView];
    self.keyImageView.frame = CGRectMake(10, self.centerView.frame.size.height-50+(50-self.keyImageView.image.size.height)/2, self.keyImageView.image.size.width, self.keyImageView.image.size.height);
    self.keyImageView.hidden = YES;
    
    [self.centerView addSubview:self.keylabel];
    CGSize size = [self.keylabel sizeThatFits:CGSizeMake(200, 30)];
    self.keylabel.frame = CGRectMake(CGRectGetMaxX(self.keyImageView.frame)+5,self.centerView.frame.size.height-50, size.width, 50);
    
    [self.centerView addSubview:self.circleImageVIew];
    self.circleImageVIew.frame = CGRectMake(CGRectGetMaxX(self.keylabel.frame)+5, self.centerView.frame.size.height-45, 40, 40);
    self.circleImageVIew.hidden = YES;
    
    [self.centerView addSubview:self.codeImageView];
    self.codeImageView.frame = CGRectMake(0,50, self.centerView.frame.size.width, self.centerView.frame.size.height-50-50);
    
    [self.centerView addSubview:self.topLine];
    self.topLine.hidden = YES;
    
    [self.centerView addSubview:self.bottomLine];
    self.bottomLine.hidden = YES;
    
    [self getData];
    
}
- (void)showLoadAnimation{
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.centerView addSubview:self.loadImageView];
        self.loadImageView.frame = CGRectMake((self.centerView.frame.size.width-self.loadImageView.image.size.width)/2,(self.centerView.frame.size.height/2-self.loadImageView.image.size.height) , self.loadImageView.image.size.width, self.loadImageView.image.size.height);
        [self.centerView addSubview:self.hudLabel];
        self.hudLabel.frame = CGRectMake((self.centerView.frame.size.width-100)/2,self.centerView.frame.size.height/2,100, 30);
        
        CABasicAnimation *animation =  [CABasicAnimation animationWithKeyPath:@"transform.rotation.z"];
        animation.fromValue = [NSNumber numberWithFloat:0.f];
        animation.toValue =  [NSNumber numberWithFloat: M_PI *2];
        animation.duration  = 1;
        animation.autoreverses = NO;
        animation.fillMode =kCAFillModeForwards;
        animation.repeatCount = MAXFLOAT;
        [self.loadImageView.layer addAnimation:animation forKey:@"rate"];
    });
}
- (void)stopLoadAnimation{
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.loadImageView.layer removeAnimationForKey:@"rate"];
        [self.loadImageView removeFromSuperview];
        self.loadImageView = nil;
        [self.hudLabel removeFromSuperview];
        self.hudLabel = nil;
    });
}
- (void)clickTap{
    dispatch_async(dispatch_get_main_queue(), ^{
        NSArray *viewArray = [self.centerView subviews];
        for (UIView *view in viewArray) {
            [view removeFromSuperview];
        }
        self.codeImageView.image = nil;
        
        [self.backView removeFromSuperview];
        self.backView = nil;
        
        [self.centerView removeFromSuperview];
        self.centerView = nil;
        
        [self.codeImageView removeFromSuperview];
        self.circleImageVIew = nil;
        
        [self.keyImageView removeFromSuperview];
        self.keyImageView = nil;
        
        [self.circleImageVIew removeFromSuperview];
        self.circleImageVIew = nil;
        
        [self.titleLabel removeFromSuperview];
        self.titleLabel = nil;
        
        [self.keylabel removeFromSuperview];
        self.keylabel = nil;
        
        if (self.delegate && [self.delegate respondsToSelector:@selector(verifyCodeCloseWindow)]) {
            [self.delegate verifyCodeCloseWindow];
        }
    });
}
- (void)setColor:(UIColor *)color{
    _color = color;
}
- (void)setAlpha:(CGFloat)alpha{
    _alpha = alpha;
}
- (void)setFrame:(CGRect)frame{
    _frame = frame;
}

- (BOOL)gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldReceiveTouch:(UITouch *)touch
{
    if ([touch.view isDescendantOfView:self.centerView]) {
        return NO;
    }
    return YES;
}
- (void)getData{
    self.titleLabel.text = @"";
    self.keylabel.text = @"";
    self.circleImageVIew.hidden = YES;
    self.keyImageView.hidden = YES;
    self.topLine.hidden = YES;
    self.bottomLine.hidden = YES;
    self.codeImageView.image = nil;
    NSArray *viewArray = [self.centerView subviews];
    for (UIView *view in viewArray) {
        if (view.tag >= 100) {
            [view removeFromSuperview];
        }
    }
    [self showLoadAnimation];
    NSURL *url = [NSURL URLWithString:[NSString stringWithFormat:@"https://api.adfortest.com/m/%@",self.appKey]];
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:url];
    request.timeoutInterval = self.timeoutInterval;
    [request setValue:self.clientToken.length == 0?@"":self.clientToken forHTTPHeaderField:@"clientToken"];
    NSURLSession *session = [NSURLSession sharedSession];
    NSURLSessionDataTask *dataTask = [session dataTaskWithRequest:request completionHandler:^(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error) {
        if (error) {
            dispatch_async(dispatch_get_main_queue(), ^{
                if (self.delegate && [self.delegate respondsToSelector:@selector(verifyCodeNetError:)]) {
                    [self.delegate verifyCodeNetError:error];
                }
            });
            [self stopLoadAnimation];
            [self showAlertMessage:@"网络超时，点击重试"];
        }else{
            NSError *jsonError;
            id json = [NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingAllowFragments error:&jsonError];
            if (jsonError) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    if (self.delegate && [self.delegate respondsToSelector:@selector(verifyCodeNetError:)]) {
                        [self.delegate verifyCodeNetError:jsonError];
                    }
                    [self stopLoadAnimation];
                    [self showAlertMessage:@"数据解析错误，点击重试"];
                });
            }else{
                NSDictionary *dic = (NSDictionary *)json;
                NSInteger code = [[dic valueForKey:@"code"] integerValue];
                if (code == 200) {
                    //成功
                    [self downloadImageWithImageDic:dic];
                }else if (code == 403){
                    dispatch_async(dispatch_get_main_queue(), ^{
                        if (self.delegate && [self.delegate respondsToSelector:@selector(verifyCodeInitFailed:)]) {
                            [self.delegate verifyCodeInitFailed:[[NSString alloc]initWithData:data encoding:NSUTF8StringEncoding]];
                        }
                        [self stopLoadAnimation];
                        [self showAlertMessage:[dic valueForKey:@"msg"]];
                    });
                }else{
                    dispatch_async(dispatch_get_main_queue(), ^{
                        if (self.delegate && [self.delegate respondsToSelector:@selector(verifyCodeInitFailed:)]) {
                            [self.delegate verifyCodeInitFailed:[[NSString alloc]initWithData:data encoding:NSUTF8StringEncoding]];
                        }
                        [self stopLoadAnimation];
                        [self showAlertMessage:[dic valueForKey:@"msg"]];
                    });
                }
            }
        }
    }];
    [dataTask resume];
}
- (void)downloadImageWithImageDic:(NSDictionary *)dict{
    NSDictionary *dic = [dict valueForKey:@"data"];
    self.clientToken = [dic valueForKey:@"clientToken"];
    NSString *imageURL = [dic valueForKey:@"img"];
    NSURL *url = [NSURL URLWithString:imageURL];
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:url];
    request.timeoutInterval = self.timeoutInterval;
    NSURLSession *session = [NSURLSession sharedSession];
    NSURLSessionDataTask *dataTask = [session dataTaskWithRequest:request completionHandler:^(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error) {
        if (error) {
            dispatch_async(dispatch_get_main_queue(), ^{
                if (self.delegate && [self.delegate respondsToSelector:@selector(verifyCodeNetError:)]) {
                    [self.delegate verifyCodeNetError:error];
                }
                [self stopLoadAnimation];
                [self showAlertMessage:@"数据请求错误，点击重试"];
            });
        }else{
            dispatch_async(dispatch_get_main_queue(), ^{
                [self stopLoadAnimation];
                self.codeImageView.image = [UIImage imageWithData:data];
                
                self.titleLabel.text = [dic valueForKey:@"title"];
                NSString *key1 = [dic valueForKey:@"ask"];
                NSString *key2 = @"气泡";
                NSMutableAttributedString *str = [[NSMutableAttributedString alloc] initWithString:[NSString stringWithFormat:@"将图中%@对应的%@拖拽到括号内",key1,key2]];
                [str addAttribute:NSForegroundColorAttributeName value:[UIColor colorWithRed:243/255.0 green:104/255.0 blue:161/255.0 alpha:1.0] range:NSMakeRange(3, key1.length)];
                [str addAttribute:NSForegroundColorAttributeName value:[UIColor colorWithRed:243/255.0 green:104/255.0 blue:161/255.0 alpha:1.0] range:NSMakeRange(3+key1.length+3, key2.length)];
                self.keylabel.attributedText = str;
//                CGSize size = [self.keylabel sizeThatFits:CGSizeMake(200, 30)];
//                self.keylabel.frame = CGRectMake(CGRectGetMaxX(self.keyImageView.frame)+5,self.centerView.frame.size.height-50, size.width, 50);
                self.keylabel.frame = CGRectMake(CGRectGetMaxX(self.keyImageView.frame)+5,self.centerView.frame.size.height-50, self.centerView.frame.size.width-(10+self.keyImageView.image.size.width+5+5+50), 50);
                self.circleImageVIew.frame = CGRectMake(CGRectGetMaxX(self.keylabel.frame)+5, self.centerView.frame.size.height-46, 42, 42);
                
                self.topLine.frame = CGRectMake(CGRectGetMaxX(self.keylabel.frame)+5+13.5, self.centerView.frame.size.height-46, 15, 2);
                self.bottomLine.frame = CGRectMake(CGRectGetMaxX(self.keylabel.frame)+5+13.5, self.centerView.frame.size.height-6, 15, 2);
                
                self.topLine.hidden = NO;
                self.bottomLine.hidden = NO;
                self.keyImageView.hidden = NO;
                self.circleImageVIew.hidden = NO;
                self.rightPoint = CGPointMake(self.circleImageVIew.frame.origin.x+20, self.circleImageVIew.frame.origin.y+20);
                [self.pointArray removeAllObjects];
                
                if (self.codeImageView.frame.size.width/self.codeImageView.frame.size.height > self.codeImageView.image.size.width/self.codeImageView.image.size.height) {
                    //以高为准
                    CGFloat scale = self.codeImageView.frame.size.height/self.codeImageView.image.size.height;
                    NSArray *array = [dic valueForKey:@"points"];
                    CGFloat width = (self.codeImageView.frame.size.width- self.codeImageView.image.size.width*scale)/2;
                    for (int i = 0; i<array.count; i++) {
                        NSDictionary *point = array[i];
                        CGFloat x = [[point valueForKey:@"X"] floatValue] *scale;
                        CGFloat y = [[point valueForKey:@"Y"] floatValue] *scale;
                        
                        UIImageView *tapImageView = [[UIImageView alloc]init];
                        UIImage *image = [UIImage imageWithData:[NSData dataWithContentsOfURL:[NSURL URLWithString:[point valueForKey:@"bubble"]]]];
                        tapImageView.image = [self image:image withTintColor:[self colorWithHexString:[point valueForKey:@"color"]]];
                        tapImageView.userInteractionEnabled = YES;
                        tapImageView.tag = 100+i;
                        [self.centerView addSubview:tapImageView];
                        tapImageView.frame = CGRectMake(x+width-15, y-15+50, 30, 30);
                        UIPanGestureRecognizer *panGesture = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(panAction:)];
                        [tapImageView addGestureRecognizer:panGesture];
                        
                        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                            CATransition *animation = [CATransition animation];
                            [animation setDuration:0.65f];
                            [animation setType:kCATransitionFade];
                            animation.removedOnCompletion = YES;
                            [tapImageView.layer addAnimation:animation forKey:@"transition"];
                            [tapImageView.layer removeAnimationForKey:@"transition"];
                        });
                        
                    }
                }else{
                    //以宽为准
                    CGFloat scale = self.codeImageView.frame.size.width/self.codeImageView.image.size.width;
                    CGFloat height = (self.codeImageView.frame.size.height- self.codeImageView.image.size.height*scale)/2;
                    NSArray *array = [dic valueForKey:@"points"];
                    for (int i = 0; i<array.count; i++) {
                        NSDictionary *point = array[i];
                        CGFloat x = [[point valueForKey:@"X"] floatValue] *scale;
                        CGFloat y = [[point valueForKey:@"Y"] floatValue] *scale;
                        
                        UIImageView *tapImageView = [[UIImageView alloc]init];
                        UIImage *image = [UIImage imageWithData:[NSData dataWithContentsOfURL:[NSURL URLWithString:[point valueForKey:@"bubble"]]]];
                        tapImageView.tag = 100+i;
                        tapImageView.image = [self image:image withTintColor:[self colorWithHexString:[point valueForKey:@"color"]]];
                        tapImageView.userInteractionEnabled = YES;
                        [self.centerView addSubview:tapImageView];
                        tapImageView.frame = CGRectMake(x-15, y+height-15+50, 30, 30);
                        UIPanGestureRecognizer *panGesture = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(panAction:)];
                        [tapImageView addGestureRecognizer:panGesture];
                        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                            CATransition *animation = [CATransition animation];
                            [animation setDuration:0.65f];
                            [animation setType:kCATransitionFade];
                            animation.removedOnCompletion = YES;
                            [tapImageView.layer addAnimation:animation forKey:@"transition"];
                            [tapImageView.layer removeAnimationForKey:@"transition"];
                        });
                    }
                }
                
            });
        }
    }];
    [dataTask resume];
}

- (void)panAction:(UIPanGestureRecognizer *)recognizer {
    CGPoint translation = [recognizer translationInView:self.codeImageView];
    CGPoint newCenter = CGPointMake(recognizer.view.center.x+ translation.x,
                                    recognizer.view.center.y + translation.y);//    限制屏幕范围：
    newCenter.y = MAX(recognizer.view.frame.size.height/2, newCenter.y);
    newCenter.y = MIN(self.centerView.frame.size.height - recognizer.view.frame.size.height/2,  newCenter.y);
    newCenter.x = MAX(recognizer.view.frame.size.width/2, newCenter.x);
    newCenter.x = MIN(self.centerView.frame.size.width - recognizer.view.frame.size.width/2,newCenter.x);
    recognizer.view.center = newCenter;
    [recognizer setTranslation:CGPointZero inView:self.codeImageView];
    
    if (recognizer.state == UIGestureRecognizerStateBegan) {
        //第一个点
        NSMutableDictionary *dic = [NSMutableDictionary dictionary];
        [dic setObject:@(recognizer.view.center.x) forKey:@"x"];
        [dic setObject:@(recognizer.view.center.y) forKey:@"y"];
        [dic setObject:@(0)forKey:@"t"];
        [dic setObject:@([[NSDate date] timeIntervalSince1970]*1000) forKey:@"date"];
        [self.pointArray addObject:dic];
    }else if (recognizer.state == UIGestureRecognizerStateChanged){
        [self getPointDicWithPoint:CGPointMake(recognizer.view.center.x, recognizer.view.center.y)];
    }else if (recognizer.state == UIGestureRecognizerStateEnded){
        
        [self getPointDicWithPoint:CGPointMake(recognizer.view.center.x, recognizer.view.center.y)];
        CGPoint point1 = self.rightPoint;
        CGPoint point2 = newCenter;
        CGFloat deltaX = fabs(point1.x-point2.x);
        CGFloat deltaY = fabs(point1.y-point2.y);
        CGFloat distance = sqrt(deltaX*deltaX+deltaY*deltaY);
        
        if (distance > 10) {
            //不正确
            [self getData];
        }else{
            //正确
            for (NSMutableDictionary *dic in self.pointArray) {
                [dic removeObjectForKey:@"date"];
            }
            //请求验证
            [self postVerifyCodeWithArray:self.pointArray appkey:self.appKey key:[NSString stringWithFormat:@"%ld",recognizer.view.tag-100]];
        }
    }
}
- (void)postVerifyCodeWithArray:(NSArray *)array appkey:(NSString *)appkey key:(NSString *)key{
    NSURL *url = [NSURL URLWithString:[NSString stringWithFormat:@"https://api.adfortest.com/m/%@/%@",self.appKey,key]];
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:url];
    request.HTTPMethod = @"POST";
    request.timeoutInterval = self.timeoutInterval;
    request.HTTPBody = [[NSString stringWithFormat:@"data=%@",array] dataUsingEncoding:NSUTF8StringEncoding];
    [request setValue:self.clientToken.length == 0?@"":self.clientToken forHTTPHeaderField:@"clientToken"];
    NSURLSession *session = [NSURLSession sharedSession];
    NSURLSessionDataTask *dataTask = [session dataTaskWithRequest:request completionHandler:^(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error) {
        if (error) {
            dispatch_async(dispatch_get_main_queue(), ^{
                if (self.delegate && [self.delegate respondsToSelector:@selector(verifyCodeNetError:)]) {
                    [self.delegate verifyCodeNetError:error];
                }
                [self getData];
            });
        }else{
            NSError *jsonError;
            id json = [NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingAllowFragments error:&jsonError];
            if (jsonError) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    if (self.delegate && [self.delegate respondsToSelector:@selector(verifyCodeNetError:)]) {
                        [self.delegate verifyCodeNetError:jsonError];
                    }
                    [self getData];
                });
            }else{
                dispatch_async(dispatch_get_main_queue(), ^{
                    NSDictionary *dic = (NSDictionary *)json;
                    NSInteger code = [[dic valueForKey:@"code"] integerValue];
                    if (code == 200) {
                        if (self.delegate && [self.delegate respondsToSelector:@selector(verifyCodeValidateFinish:message:)]) {
                            [self.delegate verifyCodeValidateFinish:YES message:@"验证成功"];
                        }
                        [self clickTap];
                    }else{
                        if (self.delegate && [self.delegate respondsToSelector:@selector(verifyCodeValidateFinish:message:)]) {
                            [self.delegate verifyCodeValidateFinish:NO message:@"验证失败"];
                        }
                        [self getData];
                    }
                });
            }
        }
    }];
    [dataTask resume];
}

- (void)showAlertMessage:(NSString *)meassage{
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.centerView addSubview:self.alertImageView];
        self.alertImageView.frame = CGRectMake((self.centerView.frame.size.width-self.alertImageView.image.size.width)/2,(self.centerView.frame.size.height/2-self.alertImageView.image.size.height) , self.alertImageView.image.size.width, self.alertImageView.image.size.height);
        [self.centerView addSubview:self.alertlabel];
        self.alertlabel.text = meassage;
        self.alertlabel.frame = CGRectMake((self.centerView.frame.size.width-200)/2,self.centerView.frame.size.height/2,200, 50);
    });
}
- (void)hideAletMessage{
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.alertImageView removeFromSuperview];
        self.alertImageView = nil;
        [self.alertlabel removeFromSuperview];
        self.alertlabel = nil;
    });
}
- (UIView *)backView{
    if (!_backView) {
        _backView = [[UIView alloc]init];
        _backView.backgroundColor = [UIColor clearColor];
        UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc]initWithTarget:self action:@selector(clickTap)];
        tap.numberOfTapsRequired = 1;
        tap.delegate = self;
        [_backView addGestureRecognizer:tap];
    }
    return _backView;
}
- (UIView *)centerView{
    if (!_centerView) {
        _centerView = [[UIView alloc]init];
        _centerView.backgroundColor = [UIColor whiteColor];
    }
    return _centerView;
}
- (UILabel *)titleLabel{
    if (!_titleLabel) {
        _titleLabel = [[UILabel alloc]init];
        _titleLabel.textColor = [UIColor colorWithRed:243/255.0 green:104/255.0 blue:161/255.0 alpha:1.0];
        _titleLabel.font = [UIFont systemFontOfSize:16];
        _titleLabel.numberOfLines = 2;
        _titleLabel.textAlignment = NSTextAlignmentLeft;
    }
    return _titleLabel;
}

- (UIImageView *)keyImageView{
    if (!_keyImageView) {
        _keyImageView = [[UIImageView alloc]init];
        _keyImageView.image = [UIImage imageNamed:@"SecurityCodeSDK.bundle/ico_ask.png"];//[UIImage imageNamed:@"ico_ask"];
    }
    return _keyImageView;
}

- (UILabel *)keylabel{
    if (!_keylabel) {
        _keylabel = [[UILabel alloc]init];
        _keylabel.textColor = [UIColor blackColor];
//        _keylabel.font = [UIFont systemFontOfSize:12];
        _keylabel.adjustsFontSizeToFitWidth = YES;
        _keylabel.textAlignment = NSTextAlignmentLeft;
    }
    return _keylabel;
}

- (UIImageView *)loadImageView{
    if (!_loadImageView) {
        _loadImageView = [[UIImageView alloc]init];
//        _loadImageView.image = [UIImage imageNamed:@"ico_loading"];
        _loadImageView.image = [UIImage imageNamed:@"SecurityCodeSDK.bundle/ico_loading.png"];
    }
    return _loadImageView;
}

- (UILabel *)hudLabel{
    if (!_hudLabel) {
        _hudLabel = [[UILabel alloc]init];
        _hudLabel.textColor = [UIColor colorWithRed:153/255.0 green:162/255.0 blue:170/255.0 alpha:1.0];
        _hudLabel.font = [UIFont systemFontOfSize:15];
        _hudLabel.textAlignment = NSTextAlignmentCenter;
        _hudLabel.text = @"加载中";
    }
    return _hudLabel;
}
- (UIImageView *)codeImageView{
    if (!_codeImageView) {
        _codeImageView = [[UIImageView alloc]init];
        _codeImageView.contentMode = UIViewContentModeScaleAspectFit;
        _codeImageView.userInteractionEnabled = YES;
    }
    return _codeImageView;
}
- (UIImageView *)circleImageVIew{
    if (!_circleImageVIew) {
        _circleImageVIew = [[UIImageView alloc]init];
//        _circleImageVIew.layer.cornerRadius = 21;
//        _circleImageVIew.layer.borderColor = [UIColor orangeColor].CGColor;
         _circleImageVIew.layer.borderColor = [UIColor colorWithRed:95/255.0 green:151/255.0 blue:200/255.0 alpha:1.0].CGColor;
        _circleImageVIew.layer.borderWidth = 2;
    }
    return _circleImageVIew;
}
- (UIView *)topLine{
    if (!_topLine) {
        _topLine = [UIView new];
        _topLine.backgroundColor = [UIColor whiteColor];
    }
    return _topLine;
}
- (UIView *)bottomLine{
    if (!_bottomLine) {
        _bottomLine = [UIView new];
        _bottomLine.backgroundColor = [UIColor whiteColor];
    }
    return _bottomLine;
}

- (UIImageView *)alertImageView{
    if (!_alertImageView) {
        _alertImageView = [[UIImageView alloc]init];
        _alertImageView.image = [UIImage imageNamed:@"SecurityCodeSDK.bundle/ico_error.png"];//[UIImage imageNamed:@"ico_error"];
        UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc]initWithTarget:self action:@selector(clickRetry)];
        _alertImageView.userInteractionEnabled = YES;
        tap.numberOfTapsRequired = 1;
        [_alertImageView addGestureRecognizer:tap];
    }
    return _alertImageView;
}

- (UILabel *)alertlabel{
    if (!_alertlabel) {
        _alertlabel = [[UILabel alloc]init];
        _alertlabel.textColor = [UIColor colorWithRed:153/255.0 green:162/255.0 blue:170/255.0 alpha:1.0];
        _alertlabel.font = [UIFont systemFontOfSize:15];
        _alertlabel.textAlignment = NSTextAlignmentCenter;
        _alertlabel.text = @"";
        _alertlabel.numberOfLines = 2;
    }
    return _alertlabel;
}

- (NSMutableArray *)pointArray{
    if (!_pointArray) {
        _pointArray = [NSMutableArray array];
    }
    return _pointArray;
}
- (void)clickRetry{
    [self hideAletMessage];
    [self getData];
}
- (void)getPointDicWithPoint:(CGPoint)point{
    NSMutableArray *dict = [self.pointArray lastObject];
    long long time = [[dict valueForKey:@"date"] longLongValue];
    NSMutableDictionary *dic = [NSMutableDictionary dictionary];
    [dic setObject:@(point.x) forKey:@"x"];
    [dic setObject:@(point.y) forKey:@"y"];
    [dic setObject:@([[NSDate date] timeIntervalSince1970]*1000-time) forKey:@"t"];
    [dic setObject:@([[NSDate date] timeIntervalSince1970]*1000) forKey:@"date"];
    [self.pointArray addObject:dic];
}
- (UIImage *)image:(UIImage *)image withTintColor:(UIColor *)color {
    UIImage *img = [image imageWithRenderingMode:UIImageRenderingModeAlwaysTemplate];
    UIGraphicsBeginImageContextWithOptions(img.size, NO, img.scale);
    [color set];
    [img drawInRect:CGRectMake(0, 0, img.size.width, img.size.height)];
    img = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    
    return img;
}
- (UIColor *) colorWithHexString: (NSString *)hexString
{
    NSString *cString = [[hexString stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]] uppercaseString];
    
    //hexString应该6到8个字符
    if ([cString length] < 6) {
        return [UIColor clearColor];
    }
    
    //如果hexString 有@"0X"前缀
    if ([cString hasPrefix:@"0X"])
        cString = [cString substringFromIndex:2];
    
    //如果hexString 有@"#""前缀
    if ([cString hasPrefix:@"#"])
        cString = [cString substringFromIndex:1];
    if ([cString length] != 6)
        return [UIColor clearColor];
    
    //RGB转换
    NSRange range;
    range.location = 0;
    range.length = 2;
    
    //R
    NSString *rString = [cString substringWithRange:range];
    
    //G
    range.location = 2;
    NSString *gString = [cString substringWithRange:range];
    
    //B
    range.location = 4;
    NSString *bString = [cString substringWithRange:range];
    
    //
    unsigned int r, g, b;
    [[NSScanner scannerWithString:rString] scanHexInt:&r];
    [[NSScanner scannerWithString:gString] scanHexInt:&g];
    [[NSScanner scannerWithString:bString] scanHexInt:&b];
    
    return [UIColor colorWithRed:((float) r / 255.0f) green:((float) g / 255.0f) blue:((float) b / 255.0f) alpha:1.0f];
}

@end

